from setuptools import setup, find_packages

setup(
    name="annie_lappd",
    version="0.1",
    packages=find_packages(),
    author="Marvin Ascencio-Sosa",
    author_email="mascenci@fnal.gov",
    description="A package that processes the ASCII data generated by the ACDC boards.",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url="https://github.com/marvlad/annie_lappd/tree/main",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'pandas>=1.0',          # Specify the version range if necessary
        'numpy>=1.18',          # Same for numpy
        'matplotlib>=3.0',      # For plotting
        'tqdm>=4.0',            # For progress bars
    ],
)

